import logging
from .const import *

_LOGGER = logging.getLogger(__name__)
_LOGGER.setLevel(logging.DEBUG)


class Connection:

    def __init__(self, session,):
        self._websession = session
        self._token = None

    def set_token(self, token):
        self._token = token

    def __get_auth_headers(self):
        _LOGGER.debug("Goldenergy API AuthHeaders")
        headers = {**DEFAULT_HEADERS}
        if self._token is not None:
            headers[AUTH_HEADER] = AUTH_TOKEN_PREFIX + self._token
        return headers

    async def api_request(self, url: str, method="get", data=None, params=None):
        async with getattr(self._websession, method)(url, headers=self.__get_auth_headers(), json=data,
                                                     params=params) as response:
            try:
                if response.status == 200 and response.content_type == JSON_CONTENT:
                    return await response.json()
                else:
                    raise Exception("HTTP Request Error: %s", str(response.status) + " " + str(response.content_type))
            except Exception as err:
                _LOGGER.error("API request error: %s", err)
                return None



from dataclasses import dataclass
import datetime


@dataclass(init=True, repr=True)
class ElectronicInvoice:
    email: str
    activationDate: datetime.datetime
    active: bool

    @classmethod
    def from_dict(cls, data):
        return cls(
            email=data.get("email"),
            activationDate=data.get("activationDate"),
            active=data.get("active")
        )



from dataclasses import dataclass
import datetime
from .meter import Meter
from ..const import GAS, ELECTRICITY


@dataclass(init=True, repr=True)
class Consumption:
    date: datetime.datetime
    meter: Meter
    realM3: int
    estimatedM3: int
    realKWh: int
    estimatedKWH: int
    energy_type: str

    @classmethod
    def from_dict(cls, data):
        energies = data.get("energies")
        consumptions = {}
        for energy in energies:
            energy_type = GAS if energy.get("energyType") == 0 else ELECTRICITY
            consumption = cls(
                date=data.get("date"),
                meter=Meter.from_dict(energy.get("meters")[0].get("meter")),
                realM3=energy.get("meters")[0].get("realM3"),
                estimatedM3=energy.get("meters")[0].get("estimatedM3"),
                realKWh=energy.get("meters")[0].get("realKWh"),
                estimatedKWH=energy.get("meters")[0].get("estimatedKWH"),
                energy_type=energy_type
            )
            consumptions[energy_type] = consumption

        return consumptions




from dataclasses import dataclass


@dataclass(init=True, repr=True)
class PaymentOwner:
    customerNo: str
    name: str
    nif: int
    phone: str
    mobile: str
    email: str
    isLastPaymentOwner: bool

    @classmethod
    def from_dict(cls, data):
        if data is None:
            return None

        return cls(
            customerNo=data.get("customerNo"),
            name=data.get("name"),
            nif=data.get("nif"),
            phone=data.get("phone"),
            mobile=data.get("mobile"),
            email=data.get("email"),
            isLastPaymentOwner=data.get("isLastPaymentOwner")
        )


from dataclasses import dataclass
import datetime


@dataclass(init=True, repr=True)
class DirectDebit:
    activationDate: datetime.datetime
    iban: str
    adc: int
    maximumLimit: str

    @classmethod
    def from_dict(cls, data):
        return cls(
            activationDate=data.get("activationDate"),
            iban=data.get("iban"),
            adc=data.get("adc"),
            maximumLimit=data.get("maximumLimit")
        )


from dataclasses import dataclass
import datetime

from .electronic_invoice import ElectronicInvoice
from .payment_owner import PaymentOwner
from .last_invoice import LastInvoice
from .direct_debit import DirectDebit
from .billing_address import BillingAddress
from .electricity_product import ElectricityProduct
from .gas_product import GasProduct
from .electricity import Electricity
from .gas import Gas


@dataclass(init=True, repr=True)
class Contract:
    contractNo: str
    initDate: datetime.datetime
    nextReadingDate: datetime.datetime
    alias: str
    paymentOwner: PaymentOwner
    contractOwner: PaymentOwner
    electronicInvoice: ElectronicInvoice
    directDebit: DirectDebit
    electricity: Electricity
    gas: Gas
    billingAddress: BillingAddress
    consumptionPointAddress: BillingAddress
    lastInvoice: LastInvoice
    mgmVoucherCode: str
    balanceAmount: float
    electricityProductList: [ElectricityProduct]
    gasProductList: [GasProduct]

    @classmethod
    def from_dict(cls, data):
        return cls(
            contractNo=data.get("contractNo"),
            initDate=data.get("initDate"),
            nextReadingDate=data.get("nextReadingDate"),
            alias=data.get("alias"),
            paymentOwner=PaymentOwner.from_dict(data.get("paymentOwner")),
            electronicInvoice=ElectronicInvoice.from_dict(data.get("electronicInvoice")),
            contractOwner=PaymentOwner.from_dict(data.get("contractOwner")),
            directDebit=DirectDebit.from_dict(data.get("directDebit")),
            electricity=Electricity.from_dict(data.get("electricity")),
            gas=Gas.from_dict(data.get("gas")),
            billingAddress=BillingAddress.from_dict(data.get("billingAddress")),
            consumptionPointAddress=BillingAddress.from_dict(data.get("consumptionPointAddress")),
            lastInvoice=LastInvoice.from_dict(data.get("lastInvoice")),
            mgmVoucherCode=data.get("mgmVoucherCode"),
            balanceAmount=data.get("balanceAmount"),
            electricityProductList=[
                ElectricityProduct.from_dict(electricity_product) for electricity_product in data.get("electricityProductList")
            ],
            gasProductList=[
                GasProduct.from_dict(gas_product) for gas_product in data.get("gasProductList")
            ]
        )



from dataclasses import dataclass
from .meter import Meter


@dataclass(init=True, repr=True)
class Electricity:
    energyType: int
    cpe: str
    power: str
    tariffType: int
    selfConsumption: bool
    status: int
    meter: Meter

    @classmethod
    def from_dict(cls, data):
        return cls(
            energyType=data.get("energyType"),
            cpe=data.get("cpe"),
            power=data.get("power"),
            tariffType=data.get("tariffType"),
            selfConsumption=data.get("selfConsumption"),
            status=data.get("status"),
            meter=Meter.from_dict(data.get("meter"))
        )


from .contract import Contract
from .consumption import Consumption
from .electronic_invoice import ElectronicInvoice
from .payment_owner import PaymentOwner
from .last_invoice import LastInvoice
from .direct_debit import DirectDebit
from .meter import Meter
from .billing_address import BillingAddress
from .electricity_product import ElectricityProduct
from .gas_product import GasProduct


import datetime
from dataclasses import dataclass


@dataclass(init=True, repr=True)
class Meter:
    meterNo: str
    serialNo: str
    initDate: datetime.datetime
    smartMeter: bool

    @classmethod
    def from_dict(cls, data):
        return cls(
            meterNo=data.get("meterNo"),
            serialNo=data.get("serialNo"),
            initDate=data.get("initDate"),
            smartMeter=data.get("smartMeter"),
        )


from dataclasses import dataclass
from .meter import Meter


@dataclass(init=True, repr=True)
class Gas:
    energyType: int
    cui: str
    escalao: int
    status: int
    meter: Meter

    @classmethod
    def from_dict(cls, data):
        return cls(
            energyType=data.get("energyType"),
            cui=data.get("cui"),
            escalao=data.get("escalao"),
            status=data.get("status"),
            meter=Meter.from_dict(data.get("meter"))
        )


from dataclasses import dataclass
import datetime


@dataclass(init=True, repr=True)
class LastInvoice:
    entryNo: int
    customerNo: str
    contractNo: str
    dueDate: datetime.datetime
    mbReference: str
    charged: bool
    chargedDate: datetime.datetime
    amount: float
    remainingAmount: float
    billingMethodDescription: str
    documentNo: int
    postingDate: datetime.datetime
    billingPeriodInitDate: datetime.datetime
    billingPeriodEndDate: datetime.datetime

    @classmethod
    def from_dict(cls, data):
        if data is None:
            return None

        return cls(
            entryNo=data.get("entryNo"),
            customerNo=data.get("customerNo"),
            contractNo=data.get("contractNo"),
            dueDate=data.get("dueDate"),
            mbReference=data.get("mbReference"),
            charged=data.get("charged"),
            chargedDate=data.get("chargedDate"),
            amount=data.get("amount"),
            remainingAmount=data.get("remainingAmount"),
            billingMethodDescription=data.get("billingMethodDescription"),
            documentNo=data.get("documentNo"),
            postingDate=data.get("postingDate"),
            billingPeriodInitDate=data.get("billingPeriodInitDate"),
            billingPeriodEndDate=data.get("billingPeriodEndDate")
        )


ENDPOINT = "https://api-clientes.goldenergy.pt/api"
LOGIN_PATH = "/user-auth/token"

CONTRACT_PATH = "/contracts/single"
CONTRACT_LIST_PATH = "/contracts/list"
CONSUMPTIONS_PATH = "/consumptions/chart-info"

JSON_CONTENT = "application/json"
DEFAULT_HEADERS = {
    "Content-Type": "application/json; charset=utf-8"
}

AUTH_TOKEN_PREFIX = "Bearer "
AUTH_HEADER = "Authorization"

CODE_PARAM = "code"
PWD_PARAM = "password"

GAS = "GAS"
ELECTRICITY = "ELECTRICITY"



from dataclasses import dataclass
import datetime


@dataclass(init=True, repr=True)
class GasProduct:
    energyType: int
    escalao: int
    initDate: datetime.datetime
    endDate: datetime.datetime

    @classmethod
    def from_dict(cls, data):
        return cls(
            energyType=data.get("energyType"),
            escalao=data.get("escalao"),
            initDate=data.get("initDate"),
            endDate=data.get("endDate")
        )


from setuptools import setup, find_namespace_packages

setup(
    name="goldenergy",
    version="0.0.2",
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    python_requires='>=3',
    package_dir={'': 'src'},
    packages=find_namespace_packages(where='src'),
    install_requires=[
        'python-dateutil',
        'aiohttp'
    ]
)



import logging

from .connection import Connection
from .const import *
from .models import Contract, Consumption, LastInvoice

_LOGGER = logging.getLogger(__name__)
_LOGGER.setLevel(logging.DEBUG)


class Goldenergy:
    def __init__(self, session, code, password):
        self._connection = Connection(session)
        self._contract: Contract = None
        self._last_consumption = None
        self._code = code
        self._password = password

    async def login(self):
        _LOGGER.debug("Goldenergy API Login")
        url = ENDPOINT + LOGIN_PATH

        data = {
            CODE_PARAM: self._code,
            PWD_PARAM: self._password
        }

        res = await self._connection.api_request(url, method="post", data=data)
        if res is not None:
            try:
                self._connection.set_token(res["result"]["token"])
                return True
            except:
                return False
        return False

    async def get_contract(self, contract_number: str) -> Contract:
        _LOGGER.debug("Goldenergy API Contract")

        url = ENDPOINT + CONTRACT_PATH

        contract_api_response = await self._connection.api_request(url, params={"ContractNo": contract_number})
        contract = contract_api_response["result"]

        self._contract = Contract.from_dict(contract)
        return self._contract

    async def get_active_contract(self) -> Contract:
        _LOGGER.debug("Goldenergy API Contract")

        url = ENDPOINT + CONTRACT_LIST_PATH

        contracts_api_response = await self._connection.api_request(url)
        contracts = contracts_api_response["result"]

        if len(contracts["contracts"]) > 1:
            for contract in contracts["contracts"]:
                if contract["status"] == 1:
                    self._contract = Contract.from_dict(contract)
                    break

        elif len(contracts["contracts"]) == 1:
            self._contract = Contract.from_dict(contracts["contracts"][0])

        return self._contract

    async def get_last_invoice(self, contract_number: str = None) -> LastInvoice:
        _LOGGER.debug("Goldenergy API Contract")

        if contract_number is None:
            await self.get_active_contract()
            contract_number = self._contract.contractNo

        url = ENDPOINT + CONTRACT_PATH

        contract_api_response = await self._connection.api_request(url, params={"ContractNo": contract_number})
        contract = contract_api_response["result"]

        self._contract = Contract.from_dict(contract)
        return self._contract.lastInvoice

    async def get_last_consumption(self, contract_no: str = None):
        _LOGGER.debug("Goldenergy API Consumptions")

        if contract_no is None:
            await self.get_active_contract()
            contract_no = self._contract.contractNo

        url = ENDPOINT + CONSUMPTIONS_PATH

        consumptions_api_response = await self._connection.api_request(
            url,
            params={"ContractNo": contract_no}
        )
        consumption = consumptions_api_response["result"]["items"][0]

        self._last_consumption = Consumption.from_dict(consumption)
        return self._last_consumption



from dataclasses import dataclass


@dataclass(init=True, repr=True)
class BillingAddress:
    streetName: str
    number: str
    duplicator: str
    fraction: str
    postCode: str
    city: str

    @classmethod
    def from_dict(cls, data):
        return cls(
            streetName=data.get("streetName"),
            number=data.get("number"),
            duplicator=data.get("duplicator"),
            fraction=data.get("fraction"),
            postCode=data.get("postCode"),
            city=data.get("city")
        )